##############################################################################################################
#                                                                                                            #
# Code showing the analyses presented in                                                                     #
# "Investigation of age-teratment interaction in the SPACE trial using differnet statistical approaches"     #
#                                                                                                            #
##############################################################################################################

# Data that have similar properites to the SPACE data are generated 
# ATTENTION: All obtained results are based on simulated data
# and consequently differ from those in the manuscript
# 

set.seed(767)
n <- 1200
GroupSim <- sample(0:1, n, repl=T)
AgeSim <- rnorm(n, 70, 10)
hazard <- 0.002 * exp(-3*GroupSim + 0.015*AgeSim + 0.04*GroupSim*AgeSim)
EvTimeSim <- rexp(n, hazard)
CensTimeSim <- pmin(rep(24,n), rexp(n,0.005))

dat <- data.frame(ID = 1:n, Group=GroupSim, Age=AgeSim, TimeMonths = pmin(EvTimeSim, CensTimeSim), Event = as.numeric(EvTimeSim < CensTimeSim))

# load survival package to perform Cox regression analysis
library(survival)



#######################################
#                                     #
# Analysis in subgroups               #
#                                     #
#######################################

# Define subgroups
dat$Age.sub <- cut(floor(dat$Age),breaks = c(0,60,70,80,999), right = F)
table(dat$Age.sub)
table(dat$Age.sub, dat$Event)

# Generate Figure 1a
plot(survfit(Surv(TimeMonths, Event) ~ Group, data=dat, subset=as.numeric(dat$Age.sub)==1), xlab="", ylab="", 
     yaxt="n", xaxt="n", lwd=3, lty=2:1, fun="event", ylim=c(0,0.4))
  title(main="< 60 years",line=0.75, cex.main=1.75)
  mtext("Time in months", side=1, line = 3, cex=1.5)
  axis(1,at=seq(0,24,by=6),seq(0,24,by=6))
  axis(2,at=seq(0,0.4,by=0.1),paste(seq(0,40,by=10),"%",sep=""), mgp=c(5,0.8,0), las=1)
  legend("topleft", lty=1:2,lwd=3,c("1","0"), bty="n", inset=c(0.1,0.1),cex=1.3, title="Group")
  mtext("Cumulative\nevent probability",side = 2, padj = 0,  at=0.2, las=0, line = 3.1, cex=1.5)

# Cox regression model for estimation of hazard ratios within the subgroups	
  Cox <- coxph(Surv(TimeMonths,Event) ~ Group, data=dat, subset=as.numeric(Age.sub)==1)
  text(16,0.35,paste("HR=",sprintf("%.2f", round(exp(Cox$coef),2)), " (", 
       round(exp(Cox$coef - qnorm(0.975)*sqrt(Cox$var)),2), " to ",
       round(exp(Cox$coef + qnorm(0.975)*sqrt(Cox$var)),2), ")",sep=""), cex=1.3)


  
# Generate Figure 1b
plot(survfit(Surv(TimeMonths, Event) ~ Group, data=dat, subset=as.numeric(dat$Age.sub)==2), xlab="", ylab="", 
     yaxt="n", xaxt="n", lwd=3, lty=2:1, fun="event", ylim=c(0,0.4))
  title(main="60 - 69 years",line=0.75, cex.main=1.75)
  mtext("Time in months", side=1, line = 3, cex=1.5)
  axis(1,at=seq(0,24,by=6),seq(0,24,by=6))
  axis(2,at=seq(0,0.4,by=0.1),paste(seq(0,40,by=10),"%",sep=""), mgp=c(5,0.8,0), las=1)
  legend("topleft", lty=1:2,lwd=3,c("1","0"), bty="n", inset=c(0.1,0.1),cex=1.3, title="Group")
  mtext("Cumulative\nevent probability",side = 2, padj = 0,  at=0.2, las=0, line = 3.1, cex=1.5)
  
# Cox regression model for estimation of hazard ratios within the subgroups	
  Cox <- coxph(Surv(TimeMonths,Event) ~ Group, data=dat, subset=as.numeric(Age.sub)==2)
  text(16,0.35,paste("HR=",sprintf("%.2f", round(exp(Cox$coef),2)), " (", 
       round(exp(Cox$coef - qnorm(0.975)*sqrt(Cox$var)),2), " to ",
       round(exp(Cox$coef + qnorm(0.975)*sqrt(Cox$var)),2), ")",sep=""), cex=1.3)


    
# Generate Figure 1c
plot(survfit(Surv(TimeMonths, Event) ~ Group, data=dat, subset=as.numeric(dat$Age.sub)==3), xlab="", ylab="", 
     yaxt="n", xaxt="n", lwd=3, lty=2:1, fun="event", ylim=c(0,0.4))
  title(main="70 - 79 years",line=0.75, cex.main=1.75)
  mtext("Time in months", side=1, line = 3, cex=1.5)
  axis(1,at=seq(0,24,by=6),seq(0,24,by=6))
  axis(2,at=seq(0,0.4,by=0.1),paste(seq(0,40,by=10),"%",sep=""), mgp=c(5,0.8,0), las=1)
  legend("topleft", lty=1:2,lwd=3,c("1","0"), bty="n", inset=c(0.1,0.1),cex=1.3, title="Group")
  mtext("Cumulative\nevent probability",side = 2, padj = 0,  at=0.2, las=0, line = 3.1, cex=1.5)
  
# Cox regression model for estimation of hazard ratios within the subgroups	
  Cox <- coxph(Surv(TimeMonths,Event) ~ Group, data=dat, subset=as.numeric(Age.sub)==3)
  text(16,0.35,paste("HR=",sprintf("%.2f", round(exp(Cox$coef),2)), " (", 
  		 round(exp(Cox$coef - qnorm(0.975)*sqrt(Cox$var)),2), " to ",
  		 round(exp(Cox$coef + qnorm(0.975)*sqrt(Cox$var)),2), ")",sep=""), cex=1.3)


   
# Generate Figure 1d
plot(survfit(Surv(TimeMonths, Event) ~ Group, data=dat, subset=as.numeric(dat$Age.sub)==4), xlab="", ylab="", 
     yaxt="n", xaxt="n", lwd=3, lty=2:1, fun="event", ylim=c(0,0.4))
  title(main=">= 80 years",line=0.75, cex.main=1.75)
  mtext("Time in months", side=1, line = 3, cex=1.5)
  axis(1,at=seq(0,24,by=6),seq(0,24,by=6))
  axis(2,at=seq(0,0.4,by=0.1),paste(seq(0,40,by=10),"%",sep=""), mgp=c(5,0.8,0), las=1)
  legend("topleft", lty=1:2,lwd=3,c("1","0"), bty="n", inset=c(0.1,0.1),cex=1.3, title="Group")
  mtext("Cumulative\nevent probability",side = 2, padj = 0,  at=0.2, las=0, line = 3.1, cex=1.5)
  
# Cox regression model for estimation of hazard ratios within the subgroups	
  Cox <- coxph(Surv(TimeMonths,Event) ~ Group, data=dat, subset=as.numeric(Age.sub)==4)
  text(16,0.35,paste("HR=",sprintf("%.2f", round(exp(Cox$coef),2)), " (", 
       round(exp(Cox$coef - qnorm(0.975)*sqrt(Cox$var)),2), " to ",
       round(exp(Cox$coef + qnorm(0.975)*sqrt(Cox$var)),2), ")",sep=""), cex=1.3)

  
  
# Test on age-treatment interaction using categorized age
Cox.int <- coxph(Surv(TimeMonths, Event) ~ Age.sub*Group, data=dat)
anova(Cox.int)





#######################################
#                                     #
# STEPP                               #
#                                     #
#######################################

# load "stepp" package to perform STEPP analysis
library(stepp)

# Set seed to make results reproducible
set.seed(1)

# Call stepp function - documentation can be found by using "?stepp"
# "patspop": number of patients in each subpopulation
# "minpatspop": number of "overlapping" patients
# "timeest": timepoint for survival estimation
STEPP <- stepp("KM", coltrt = dat$Group, coltime=dat$TimeMonths, colcens=dat$Event, colvar=dat$Age,
               timest = 24, minpatspop = 100, patspop = 300, trts = 1:0, noperm=1000)

# Show results
stepp_print(STEPP)
# Estaimted log hazard ratio
lHR.STEPP <- STEPP@effect$Ratios[[1]]$logHR
# According standard errors
se.STEPP <- STEPP@effect$Ratios[[1]]$logHRSE
# Medians of the subpopulations
MedAlt <- STEPP@subpop@medianz



# Generate Figure 2 - plot of hazard ratio
# A pointwise 95% confidence interval is drawn in the figure for better comparison with the other methods.
# When the "stepp_plot" function is used, a simultaneous confidence interval will be drawn by default.
plot(MedAlt, lHR.STEPP, type="l", xlim=range(dat$Age), ylim=c(-2,2), lwd=2, ylab="Hazard ratio", yaxt="n", xlab="Age in years")
  abline(h=0, col="grey", lty=3)
  points(MedAlt, lHR.STEPP, pch=18,cex=1.2)
  axis(1,at=MedAlt,round(MedAlt,1),mgp=c(0,-1.5,0),tcl=0.4,font=2, cex.axis=0.6)
  lines(MedAlt, lHR.STEPP - qnorm(0.975)*se.STEPP, type="l", lwd=2, lty=2)
  lines(MedAlt, lHR.STEPP + qnorm(0.975)*se.STEPP, type="l", lwd=2, lty=2)
  axis(2,at=log(c(0.2,0.3,0.5,0.7,1,1.5,2,3,4,5,7.5)), c(0.2,0.3,0.5,0.7,1,1.5,2,3,4,5,7.5), las=1)

  

  
  
#######################################
#                                     #
# Cox regression with interaction     #
#                                     #
#######################################

Cox_with_interaction <- coxph(Surv(TimeMonths, Event) ~ Group*Age, data=dat)
summary(Cox_with_interaction)

# 95% confidence interval for interaction coefficient
round(Cox_with_interaction$coefficients[3] + c(-1,1) * qnorm(0.975) * sqrt(Cox_with_interaction$var[3,3]), 3)

# Estimated log-hazard ratio between groups for a given covariate value
lHR.Cox <- Cox_with_interaction$coef[1] + seq(min(dat$Age), max(dat$Age), length=200) * Cox_with_interaction$coef[3]

# Standard error
se.Cox <- sqrt(Cox_with_interaction$var[1,1] + seq(min(dat$Age), max(dat$Age), length=200)^2*Cox_with_interaction$var[3,3] + 
               2*seq(min(dat$Age), max(dat$Age), length=200)*Cox_with_interaction$var[1,3])

# Lower CI limit
CIlow.Cox <- Cox_with_interaction$coef[1] + seq(min(dat$Age), max(dat$Age), length=200) * Cox_with_interaction$coef[3] - 
					 	 qnorm(0.975)*sqrt(Cox_with_interaction$var[1,1] + seq(min(dat$Age), max(dat$Age), length=200)^2*Cox_with_interaction$var[3,3] + 
						 2*seq(min(dat$Age), max(dat$Age), length=200)*Cox_with_interaction$var[1,3])

# Upper CI limit
CIupp.Cox <- Cox_with_interaction$coef[1] + seq(min(dat$Age), max(dat$Age), length=200) * Cox_with_interaction$coef[3] +
	           qnorm(0.975)*sqrt(Cox_with_interaction$var[1,1] + seq(min(dat$Age), max(dat$Age), length=200)^2*Cox_with_interaction$var[3,3] + 
						 2*seq(min(dat$Age), max(dat$Age), length=200)*Cox_with_interaction$var[1,3])

plot(seq(min(dat$Age), max(dat$Age), length=200), lHR.Cox, type="l", ylim=c(min(CIlow.Cox), max(CIupp.Cox)), yaxt="n", xlab="Age in years",
		 ylab="Hazard ratio",lwd=2)
  abline(h=0, lty=3, col="grey")
  lines(seq(min(dat$Age), max(dat$Age), length=200), lHR.Cox - qnorm(0.975)*se.Cox, lty=2)
  lines(seq(min(dat$Age), max(dat$Age), length=200), lHR.Cox + qnorm(0.975)*se.Cox, lty=2)
  axis(2,at=log(c(0.2,0.3,0.5,0.7,1,1.5,2,3,4,5,7.5)), c(0.2,0.3,0.5,0.7,1,1.5,2,3,4,5,7.5), las=1)

  

  
  
#######################################
#                                     #
# MFPI													      #
#                                     #
#######################################

#######################################
# FP2 - flex1
#######################################

# Fit all 36 models with FP2 transformations to the data (without covariate-treatment interaction)
C11 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-2)) + I(Age^(-2)*log(Age)) + Group, data=dat)
C12 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-2)) + I(Age^(-1)) + Group, data=dat)
C13 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-2)) + I(Age^(-0.5)) + Group, data=dat)
C14 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-2)) + I(log(Age)) + Group, data=dat)
C15 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-2)) + I(Age^0.5) + Group, data=dat)
C16 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-2)) + I(Age^1) + Group, data=dat)
C17 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-2)) + I(Age^2) + Group, data=dat)
C18 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-2)) + I(Age^3) + Group, data=dat)

C22 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-1)) + I(Age^(-1)*log(Age)) + Group, data=dat)
C23 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-1)) + I(Age^(-0.5)) + Group, data=dat)
C24 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-1)) + I(log(Age)) + Group, data=dat)
C25 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-1)) + I(Age^0.5) + Group, data=dat)
C26 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-1)) + I(Age^1) + Group, data=dat)
C27 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-1)) + I(Age^2) + Group, data=dat)
C28 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-1)) + I(Age^3) + Group, data=dat)

C33 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-0.5)) + I(Age^(-0.5)*log(Age)) + Group, data=dat)
C34 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-0.5)) + I(log(Age)) + Group, data=dat)
C35 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-0.5)) + I(Age^0.5) + Group, data=dat)
C36 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-0.5)) + I(Age^1) + Group, data=dat)
C37 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-0.5)) + I(Age^2) + Group, data=dat)
C38 <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-0.5)) + I(Age^3) + Group, data=dat)

C44 <- coxph(Surv(TimeMonths, Event) ~ I(log(Age)) + I(log(Age)*log(Age)) + Group, data=dat)
C45 <- coxph(Surv(TimeMonths, Event) ~ I(log(Age)) + I(Age^0.5) + Group, data=dat)
C46 <- coxph(Surv(TimeMonths, Event) ~ I(log(Age)) + I(Age^1) + Group, data=dat)
C47 <- coxph(Surv(TimeMonths, Event) ~ I(log(Age)) + I(Age^2) + Group, data=dat)
C48 <- coxph(Surv(TimeMonths, Event) ~ I(log(Age)) + I(Age^3) + Group, data=dat)

C55 <- coxph(Surv(TimeMonths, Event) ~ I(Age^0.5) + I(Age^0.5*log(Age)) + Group, data=dat)
C56 <- coxph(Surv(TimeMonths, Event) ~ I(Age^0.5) + I(Age^1) + Group, data=dat)
C57 <- coxph(Surv(TimeMonths, Event) ~ I(Age^0.5) + I(Age^2) + Group, data=dat)
C58 <- coxph(Surv(TimeMonths, Event) ~ I(Age^0.5) + I(Age^3) + Group, data=dat)

C66 <- coxph(Surv(TimeMonths, Event) ~ I(Age^1) + I(Age^1 * log(Age)) + Group, data=dat)
C67 <- coxph(Surv(TimeMonths, Event) ~ I(Age^1) + I(Age^2) + Group, data=dat)
C68 <- coxph(Surv(TimeMonths, Event) ~ I(Age^1) + I(Age^3) + Group, data=dat)

C77 <- coxph(Surv(TimeMonths, Event) ~ I(Age^2) + I(Age^2*log(Age)) + Group, data=dat)
C78 <- coxph(Surv(TimeMonths, Event) ~ I(Age^2) + I(Age^3) + Group, data=dat)

C88 <- coxph(Surv(TimeMonths, Event) ~ I(Age^3) + I(Age^3*log(Age)) + Group, data=dat)



# List of transformations indicating all inestifgates combinations
TrafosFP2 <- list(c(-2,-2), c(-2,-1), c(-2,-0.5), c(-2,0), c(-2,0.5), c(-2,1), c(-2,2), c(-2,3),
 			 	          c(-1,-1), c(-1,-0.5), c(-1,0), c(-1,0.5), c(-1,1), c(-1,2), c(-1,3),
								  c(-0.5,-0.5), c(-0.5,0), c(-0.5,0.5), c(-0.5,1), c(-0.5,2), c(-0.5,3),
								  c(0,0), c(0,0.5), c(0,1), c(0,2), c(0,3),
								  c(0.5,0.5), c(0.5,1), c(0.5,2), c(0.5,3),
								  c(1,1), c(1,2), c(1,3),
								  c(2,2), c(2,3),
								 	c(3,3))



# Find model with the highest likelihood
summFP2 <- c(summary(C11)$logtest[1], summary(C12)$logtest[1], summary(C13)$logtest[1], summary(C14)$logtest[1], 
					   summary(C15)$logtest[1], summary(C16)$logtest[1], summary(C17)$logtest[1], summary(C18)$logtest[1],
						 summary(C22)$logtest[1], summary(C23)$logtest[1], summary(C24)$logtest[1], 
						 summary(C25)$logtest[1], summary(C26)$logtest[1], summary(C27)$logtest[1], summary(C28)$logtest[1],
						 summary(C33)$logtest[1], summary(C34)$logtest[1], 
						 summary(C35)$logtest[1], summary(C36)$logtest[1], summary(C37)$logtest[1], summary(C38)$logtest[1],
						 summary(C44)$logtest[1], 
						 summary(C45)$logtest[1], summary(C46)$logtest[1], summary(C47)$logtest[1], summary(C48)$logtest[1],
						 summary(C55)$logtest[1], summary(C56)$logtest[1], summary(C57)$logtest[1], summary(C58)$logtest[1],
						 summary(C66)$logtest[1], summary(C67)$logtest[1], summary(C68)$logtest[1],
						 summary(C77)$logtest[1], summary(C78)$logtest[1],
						 summary(C88)$logtest[1])

TrafosFP2[[which.max(summFP2)]]

#######################################################################
# Best model - for the given seed (530): Age^(-1) + Age^(-0.5)
#######################################################################



# Test on Interaktion:
# Generate the according transformed covariates for the model with interaction
# ATTENTION: For other data (e.g. by using another seed) the code lines will have to be adapted
dat$Age.Gr0_m1 <- dat$Age^(-1)*(dat$Group==0)
dat$Age.Gr0_m05 <- dat$Age^(-0.5)*(dat$Group==0)

dat$Age.Gr1_m1 <- dat$Age^(-1)*(dat$Group==1)
dat$Age.Gr1_m05 <- dat$Age^(-0.5)*(dat$Group==1)

Cox_FP2f1 <- coxph(Surv(TimeMonths, Event) ~ dat$Age.Gr0_m1 + dat$Age.Gr0_m05 + dat$Age.Gr1_m1 + dat$Age.Gr1_m05 + Group, data=dat)

# p value for interaction
pchisq(2*(Cox_FP2f1$loglik[2] - C22$loglik[2]), df=2, lower.tail=F)



#################################################
# FP1-flex3
#################################################

# Generate transfomred covariates to fit models with interactions
dat$Age.Gr0_m2 <- dat$Age^(-2)*(dat$Group==0)
dat$Age.Gr1_m2 <- dat$Age^(-2)*(dat$Group==1)

dat$Age.Gr0_m1 <- dat$Age^(-1)*(dat$Group==0)
dat$Age.Gr1_m1 <- dat$Age^(-1)*(dat$Group==1)

dat$Age.Gr0_m05 <- dat$Age^(-0.5)*(dat$Group==0)
dat$Age.Gr1_m05 <- dat$Age^(-0.5)*(dat$Group==1)

dat$Age.Gr0_log <- log(dat$Age)*(dat$Group==0)
dat$Age.Gr1_log <- log(dat$Age)*(dat$Group==1)

dat$Age.Gr0_p05 <- dat$Age^0.5*(dat$Group==0)
dat$Age.Gr1_p05 <- dat$Age^0.5*(dat$Group==1)

dat$Age.Gr0_p1 <- dat$Age^1*(dat$Group==0)
dat$Age.Gr1_p1 <- dat$Age^1*(dat$Group==1)

dat$Age.Gr0_p2 <- dat$Age^2*(dat$Group==0)
dat$Age.Gr1_p2 <- dat$Age^2*(dat$Group==1)

dat$Age.Gr0_p3 <- dat$Age^3*(dat$Group==0)
dat$Age.Gr1_p3 <- dat$Age^3*(dat$Group==1)



# Fit all models with interaction to the data
C1 <- coxph(Surv(TimeMonths, Event) ~ Age.Gr0_m2 + Age.Gr1_m2 + Group, data=dat)
C2 <- coxph(Surv(TimeMonths, Event) ~ Age.Gr0_m1 + Age.Gr1_m1 + Group, data=dat)
C3 <- coxph(Surv(TimeMonths, Event) ~ Age.Gr0_m05 + Age.Gr1_m05 + Group, data=dat)
C4 <- coxph(Surv(TimeMonths, Event) ~ Age.Gr0_log + Age.Gr1_log + Group, data=dat)
C5 <- coxph(Surv(TimeMonths, Event) ~ Age.Gr0_p05 + Age.Gr1_p05 + Group, data=dat)
C6 <- coxph(Surv(TimeMonths, Event) ~ Age.Gr0_p1 + Age.Gr1_p1 + Group, data=dat)
C7 <- coxph(Surv(TimeMonths, Event) ~ Age.Gr0_p2 + Age.Gr1_p2 + Group, data=dat)
C8 <- coxph(Surv(TimeMonths, Event) ~ Age.Gr0_p3 + Age.Gr1_p3 + Group, data=dat)

TrafosFP1 <- c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)

# Find the model with the highest likelihood
lt <-c(summary(C1)$logtest[1], summary(C2)$logtest[1], summary(C3)$logtest[1], summary(C4)$logtest[1],
		   summary(C5)$logtest[1], summary(C6)$logtest[1], summary(C7)$logtest[1], summary(C8)$logtest[1])

TrafosFP1[which.max(lt)]



# Find the best transformation in models without interaction
C1noI <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-2)) + Group, data=dat)
C2noI <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-1)) + Group, data=dat)
C3noI <- coxph(Surv(TimeMonths, Event) ~ I(Age^(-0.5)) + Group, data=dat)
C4noI <- coxph(Surv(TimeMonths, Event) ~ I(log(Age)) + Group, data=dat)
C5noI <- coxph(Surv(TimeMonths, Event) ~ I(Age^0.5) + Group, data=dat)
C6noI <- coxph(Surv(TimeMonths, Event) ~ I(Age^1) + Group, data=dat)
C7noI <- coxph(Surv(TimeMonths, Event) ~ I(Age^2) + Group, data=dat)
C8noI <- coxph(Surv(TimeMonths, Event) ~ I(Age^3) + Group, data=dat)

# Find the model with the highest likelihood
lnoI <- c(summary(C1noI)$logtest[1], summary(C2noI)$logtest[1], summary(C3noI)$logtest[1], summary(C4noI)$logtest[1],
					summary(C5noI)$logtest[1], summary(C6noI)$logtest[1], summary(C7noI)$logtest[1], summary(C8noI)$logtest[1])

TrafosFP1[which.max(lnoI)]



########################################################
# Define best Model - here: 
#			with Interaction: Age^2 (=C7)
#			without interaction: Age^3 (=C8noI)
########################################################

pchisq(2*(C7$loglik[2] - C8noI$loglik[2]), df=1, lower.tail=F)

dat$Age.Gr0_p2 <- dat$Age^2*(dat$Group==0)
dat$Age.Gr1_p2 <- dat$Age^2*(dat$Group==1)
Cox_FP1f3 <- coxph(Surv(TimeMonths, Event) ~ dat$Age.Gr0_p2 + dat$Age.Gr1_p2 + Group, data=dat)



#########################################
# Calculations for illustration: FP2 - flex1
#########################################

# Relevent age values for figure
PredX <- 45:95

# Transformations (ATTENTION: Have to be adapted if other best transformations are found!)
xp1 <- PredX^(-1)   
xp2 <- PredX^(-0.5)

# Estiamtion of log-hazard ratio 
lHR.FP2_flex1 <- Cox_FP2f1$coef[5] + Cox_FP2f1$coef[3]*xp1 + Cox_FP2f1$coef[4]*xp2 - (Cox_FP2f1$coef[1]*xp1 + Cox_FP2f1$coef[2]*xp2)
# Estimation of standard error for log-HR
Cov.FP2_flex1 <- Cox_FP2f1$var
se.FP2_flex1 <- sqrt(Cov.FP2_flex1[5,5] + xp1^2*Cov.FP2_flex1[3,3] + xp2^2*Cov.FP2_flex1[4,4] + xp1^2*Cov.FP2_flex1[1,1] + xp2^2*Cov.FP2_flex1[2,2] +
										 	2*xp1*Cov.FP2_flex1[5,3] + 2*xp2*Cov.FP2_flex1[5,4] - 2*xp1*Cov.FP2_flex1[5,1] - 2*xp2*Cov.FP2_flex1[5,2] +
										 	2*xp1*xp2*Cov.FP2_flex1[3,4] - 2*xp1*xp1*Cov.FP2_flex1[3,1] - 2*xp1*xp2*Cov.FP2_flex1[3,2] - 
										 	2*xp2*xp1*Cov.FP2_flex1[4,1] - 2*xp2*xp2*Cov.FP2_flex1[4,2] +
										 	2*xp1*xp2*Cov.FP2_flex1[1,2])



#########################################
# Calculations for illustration: FP1 - flex3
#########################################

# Transformation (ATTENTION: Has to be adapted if other best transformations are found!)
xp <- PredX^2   

# Estiamtion of log-hazard ratio
lHR.FP1_flex3 <- Cox_FP1f3$coef[3] + Cox_FP1f3$coef[2]*xp - Cox_FP1f3$coef[1]*xp
# Estimation of standard error for log-HR
Cov.FP1_flex3 <- Cox_FP1f3$var
se.FP1_flex3 <- sqrt(Cov.FP1_flex3[3,3] + xp^2*Cov.FP1_flex3[2,2] + xp^2*Cov.FP1_flex3[1,1] +
										 	2*xp*Cov.FP1_flex3[3,2] - 2*xp*Cov.FP1_flex3[3,1] -
										 	2*xp*xp*Cov.FP1_flex3[2,1])



#########################################
### FIGURE - MFPI
#########################################

# Estimated hazard ratio (log-scale) for the FP2-flex1 model
plot(PredX, lHR.FP2_flex1, type="n", xlim=range(PredX), ylim=c(-2,2), lwd=2, ylab="Hazard ratio", yaxt="n", xlab="Age in years")
# Confidence interval for FP2-model
  polygon(c(PredX, rev(PredX)), c(lHR.FP2_flex1 - qnorm(0.975)*se.FP2_flex1, rev(lHR.FP2_flex1 + qnorm(0.975)*se.FP2_flex1)), col="grey70", border=F)
# Estimated HR for FP2-flex1 model
  lines(PredX, lHR.FP2_flex1, col="grey30", lwd=2, lty=4)

# Estimate for FP1-flex3 model
  lines(PredX, lHR.FP1_flex3, type="l", lwd=2)
# Line of zero (HR = 1)
  abline(h=0, col="grey40", lty=3, lwd=1)
# Confidence interval for FP1-flex3 model
  lines(PredX, lHR.FP1_flex3 + qnorm(0.975)*se.FP1_flex3, type="l", lwd=2, lty=2)
  lines(PredX, lHR.FP1_flex3 - qnorm(0.975)*se.FP1_flex3, type="l", lwd=2, lty=2)

  axis(2,at=log(c(0.2,0.3,0.5,0.7,1,1.5,2,3,4,5,7.5)), c(0.2,0.3,0.5,0.7,1,1.5,2,3,4,5,7.5))

  legend("topleft", lty=c(4,1), col=c("grey30", "black"), inset=c(0.05,0.05),
	  		 c("FP2 - flex1: p = (-1, -0.5)", "FP1 - flex3: p = 2"), lwd=2, bty="n", seg.len=3)





#######################################
#                                     #
# LPLE													      #
#                                     #
#######################################

# load library lplb (can be dowbnloaded from github)
library(lplb)

# Set seed to make results reproducible
set.seed(1)
# Ten local points and only 400 bootstrap samples are used here to reduce computational burden - 
# more were considered in the analysis presented in the manuscript
lple.fit <- lplb(Surv(TimeMonths, Event) ~ Group + Age, data=dat, control = list(
						h=0.2, kernel="epanechnikov",B=400, w0=seq(0.05,0.95,by=0.10)))
lple.fit

lHR.LPLE <- lple.fit$beta_w
se.LPLE <- lple.fit$sd



# Illustration:
plot(quantile(dat$Age,prob=lple.fit$w_est), lHR.LPLE, type="l", xlim=range(dat$Age), ylim=c(-2,2), lwd=2, 
     ylab="Hazard ratio", yaxt="n", xlab="Age in years")

  abline(h=0, col="grey", lty=3)

# 95% confidence interval
  lines(quantile(dat$Age,prob=lple.fit$w_est), lple.fit$beta_w + qnorm(0.975)*se.LPLE, type="l", lwd=2, lty=2)
  lines(quantile(dat$Age,prob=lple.fit$w_est), lple.fit$beta_w - qnorm(0.975)*se.LPLE, type="l", lwd=2, lty=2)
  
  axis(2,at=log(c(0.2,0.3,0.5,0.7,1,1.5,2,3,4,5,7.5)), c(0.2,0.3,0.5,0.7,1,1.5,2,3,4,5,7.5))




  
#######################################
#                                     #
# Modified covariate approach		      #
#                                     #
#######################################
  
# load library "splines"
library(splines)
  
# Transform treatment variable to {-1,1}
dat$Treat.modCov <- (dat$Group==1) - (dat$Group==0)
  
# B-spline basis functions are used
# In the manuscript more basis function are considered 
# Number of basis functions is smaller here to reduce computational burden
# For real life applications this might have to be adapted
BS <- bs(dat$Age,df=6, intercept = F)
CC <- cbind(1,BS)

# Penalization matrix
Dmat <- matrix(0,nrow=dim(CC)[2]-2,ncol=dim(CC)[2])
for(di in 1:(dim(CC)[2]-2))
 {
 	Dmat[di,di] <- 1
 	Dmat[di,di+1] <- -2
 	Dmat[di,di+2] <- 1
 }  

  
# Function for penalized log-likelihood
loli.penal <- function(Time, Status, Covariate, Treat.mod, lambda, coeffs, D)
 {
 	# Time: Event time or last known time without an event
 	# Status: Event indicator (1=observed event, 0=censored observation)
 	# Covariate: Matrix of transformed covariate (e.g. B-spline basis functions)
 	# Treat.mod: Transformed treatment indicator (0 -> -1)
 	# lambda: smoothing parameter
 	# coeffs: coefficients --> to be estimated
 	# D: penalization matrix
 	
 	# Calculate W*
 	Cov.star <- Covariate * Treat.mod / 2
	
 	# log-likelihood
  	loli <- Status*(Cov.star %*% coeffs-
 						sapply(X=Time, function(x) log(sum(as.numeric(Time>=x) * exp(Cov.star %*% coeffs), na.rm=T))))
  	
  	# return penalized log-likelihood
  	return(sum(loli)  - 0.5 * lambda * t(matrix(coeffs,ncol=1)) %*% t(D) %*% D %*% coeffs)
  }
  
  
# Function for unpenalized log-likelihood 
# (used for calculation of effective degrees of freedom)
loli.unpenal <- function(Time, Status, Covariate, Treat.mod, coeffs)
 {
 	Cov.star <- Covariate * Treat.mod / 2
  	
 	# log-likelihood
 	loli <- Status*(Cov.star %*% coeffs-
 					sapply(X=Time, function(x) log(sum(as.numeric(Time>=x) * exp(Cov.star %*% coeffs), na.rm=T))))
 	return(sum(loli))
 }
  
optUnpenal <- optim(loli.unpenal, par = rep(0,dim(CC)[2]), Time=dat$TimeMonths,
 										Status=dat$Event, Covariate=CC, Treat.mod=dat$Treat.modCov,
 										method="BFGS",control = list(trace=1, maxit=100000, fnscale=-1),
 										hessian = T)
 
  
# Candidates for optimal smoothing parameter 
# Here candidates of 0.1, 0.1, 1, 10, 100, 1000, 10000, 100000, 1000000, 10000000 are proposed. 
# More candidates were considered in the analyses presented in the manuscript
# Other sets might be more suitable for other data or other choices of pline functions or knots!
# ATTENTION: Computation time can be very long with other settings
LAMs <-  10^(-1:7)
  
# Estimate vector of regression coefficients for all lambdas 
loop.lambda <- list()
for(run.lam in 1:length(LAMs))
 {
	loop.lambda[[run.lam]] <- 
	optim(loli.penal, par = rep(0,dim(CC)[2]), Time=dat$TimeMonths,
	      Status=dat$Event, Covariate=CC, Treat.mod=dat$Treat.modCov, D=Dmat,
  			method="BFGS",control = list(trace=1, maxit=100000, fnscale=-1),
  			lambda=LAMs[run.lam],hessian = T)
  print(run.lam)
 }
  
# Initialize objects to save predicted values, variances, effective degrees of freedom, AIC
# and maximum loglikelihood for all values of lambda
lHR.modCov <- matrix(nrow=length(LAMs),ncol=dim(dat)[1])
IMAT <- list()
IMAT.inv <- list()
eff.df <- c()
AIC <- c()
maxLL <- c()
VARmat <- matrix(nrow=length(LAMs),ncol=dim(dat)[1])
  
# loop over all lambda
for(i in 1:length(LAMs))
 {
 	# predicted log-hazard ratio for observed age values
 	lHR.modCov[i,] <- CC%*%loop.lambda[[i]]$par
	
 	# Fisher information matrix and its inverse 
 	IMAT[[i]] <- -loop.lambda[[i]]$hessian
 	IMAT.inv[[i]] <- solve(IMAT[[i]])
 	

 	# Effective degrees of freedom
 	eff.df[i] <- sum(diag(-optUnpenal$hessian %*% solve(IMAT[[i]])))
  	
  # AIC using effective df
 	AIC[i] <- 2*eff.df[i] - 2*(loop.lambda[[i]]$value)
  	
 	# Maximum log-likelihood
 	maxLL[i] <- loop.lambda[[i]]$value
  	
  # Variance for log-HR at observed age values
  for(j in 1:dim(dat)[1])
    VARmat[i,j] <- t(CC[j,]) %*% IMAT.inv[[i]] %*% CC[j,]
 }
  
  
# Find "optimal" smoothing parameter
BEST <- which.min(AIC)
LAMs[BEST]
eff.df[BEST]
plot(log(LAMs, base=10), AIC)
  
# Sort observed age values
sort.ind <- sort(dat$Age, index.return=T)$ix
  
# plot estiamted hazard ratio fog observed age values
plot(1,1, xlim=range(dat$Age), xlab="Age in years",type="n",ylim=c(-2,2), yaxt="n", ylab="Hazard ratio")
 abline(h=0,col="grey",lty=3)
 lines(dat$Age[sort.ind],lHR.modCov[BEST,sort.ind], lwd=2)
  
# Add lower and upper confidence interval limits
 lines(dat$Age[sort.ind],lHR.modCov[BEST,sort.ind] + qnorm(0.975)*sqrt(VARmat[BEST,sort.ind]),lty=2, lwd=2)
 lines(dat$Age[sort.ind],lHR.modCov[BEST,sort.ind] - qnorm(0.975)*sqrt(VARmat[BEST,sort.ind]),lty=2, lwd=2)	
 axis(2,at=log(c(0.2,0.3,0.5,0.7,1,1.5,2,3,4,5,7.5)), c(0.2,0.3,0.5,0.7,1,1.5,2,3,4,5,7.5))
  


